/** @(#)Automat.java	1.0b 30.Januar 1998 
	* Verwendungsbeispiel:
	* Automat a = new Automat();		// Automat mit Anfangszustand "S0"
	* a.v('1',"S0","S1","Eins gewhlt");
	* a.v('2',"S1","Ende","Ende gewhlt");
	* a.v('e',"S0","Ende","von Start aus Ende gewhlt");
	* a.v('n',"Ende","S0","Neustart");
	* a.v('q',"Ende","Sackgasse","Die los()-Methode kehrt jetzt zurck");
	*
	* a.los();										
	* 

*/

import java.util.Hashtable;

/** Statusklasse. Kann sich den Namen eines Zustands und alle von ihm ausgehenden Verknpfungen
	* merken. */
class S
{
	/** Name des Status */
	String name;
	/** ordnet beliebigen Eingabeobjekten eine Verknpfung zu einem Folgezustand zu */
	Hashtable verknuepfungen;
	

	/** Konstruktor
		* @param nname Name des Status */
	public S(String nname)
	{
		name = nname;
		verknuepfungen = new Hashtable();
	}
	
	/** Fgt dem Zustand eine Verknpfung zu einem anderen Zustand zu, die bei der Eingabe
		* des input-Strings durchlaufen wird, wobei dann output ausgegeben wird. 
		* @param input Eingabestring
		* @param status Status, zu dem die Verknpfung zeigt
		* @param output Zeichenkette, die beim Durchlaufen der Verbindung ausgegeben wird.
		*/
	public void neueVerknuepfung(String input,S status , String output)
	{
		// Neue Verknpfung erstellen
		Verknuepfung v = new Verknuepfung(this,status,output);
		verknuepfungen.put(input,v);  // in die Liste der Verknpfungen aufnehmen
	}
	
	/** Folgt der dem Inputstring zugeordneten Verknpfung, gibt den entsprechenden String aus
		* und gibt eine Referenz auf den Zielstatus zurck. 
		* @param input Eingabestring, dem eine Verknpfung zugeordnet sein mu
		* @return Zielstatus */
	public S geheZu(String input)
	{
		// zugeordnete Verknpfung auslesen
		Verknuepfung v = (Verknuepfung)verknuepfungen.get(input);
		// Ist input tatschlich etwas zugeordnet?
		if(v != null)
		{
			// Meldung der Verknpfung ausgeben
			System.out.println(v.output());
		  // Ziel zurckgeben
		  return v.nach();
		 }
		 else			// keine Verknpfung zugeordnet
		 {
		 	 System.out.println("Falsche Eingabe");
		 	 // Zielstatus = Ausgangsstatus
		 	 return this;
		 	}
	}
	
/** istEnde() gibt true zurck, wenn von dem aktuellen Objekt keine Verknpfungen ausgehen(eine
	* Art Sackgasse). 
	* @return true, wenn Sackgasse, sonst false. */
public boolean istEnde()
{
	// Verknpfungsliste leer?
	return verknuepfungen.size() == 0;
}

/** Gibt den Namen des Status zurck */
public String name() { return name;}
	
}

/** Diese Klasse reprsentiert eine Verknpfung zwischen zwei Zustnden. */
class Verknuepfung
{
	/** Status, von dem die Verknpfung ausgeht */
	S von;
	/** Zielstatus */
	S nach;
	/** Ausgabe, die beim Durchlaufen der Verknpfung ausgegeben wird. */
	String output;
	
	/** Konstruktor
		* @param nvon Ausgangsstatus
		* @param nnach Zielstatus
		* @param noutput Meldung beim Durchlaufen */
	public Verknuepfung(S nvon,S nnach, String noutput)
	{
		von = nvon;
		nach = nnach;
		output = noutput;
	}
	
	public String output(){	return output;}
	public S nach() { return nach;}
	public S von() {	return von;	}
}



/** Die Automatklasse verwaltet einen Graphen aus Status- und Verknpfungsobjekten */
public class Automat
{
	S s0;		/** Startstatus*/
	S aktZustand;	/** aktueller Zustand des Automaten */
	
	/** enthlt alle Zustnde, die im Automaten enthalten sind, zugeordnet zu ihren Namen */
	Hashtable alleZustaende;
		
	/** Standardkonstruktor; erzeugt einen Automaten mit einem Anfangszustand namens "S0" */
	public Automat()
	{
		this("S0");
	}
	
	/** Konstruktor; erzeugt einen Automaten mit dem bergebenen Anfangszustand
		* @param ns0 Anfangszustand des Automaten */
	public Automat(String ns0)
	{
		s0 = new S(ns0);
		alleZustaende = new Hashtable();
		// Anfangszustand in die Zustandsliste einfgen
		alleZustaende.put(s0.name(),s0);
		// Der Automat beginnt natrlich beim Anfangszustand
		aktZustand = s0;
	}
	
	/** Verknpft zwei Zustnde miteinander.
		* @param input Eingabe, bei der Verknpfung durchlaufen werden soll
		* @param von Ausgangsstatus
		* @param nach Zielstatus
		* @param output Ausgabe beim Durchlaufen der Verknpfung*/
	public void v(String input, S von, S nach, String output)
	{
		von.neueVerknuepfung(input,nach,output);
	}
	
	/** Verknpft zwei Zustnde miteinander, die durch ihre Namen referenziert werden.
		* @param input Eingabe, bei der Verknpfung durchlaufen werden soll
		* @param von Name des Ausgangsstatus
		* @param nach Name des Zielstatus
		* @param output Ausgabe beim Durchlaufen der Verknpfung*/
		
	public void v(String input, String von, String nach, String output)
	{
		 // Durch die Zustandsliste den Statusnamen die Zustandsobjekte auslesen.
		 S svon = (S)alleZustaende.get(von);
		 S snach = (S)alleZustaende.get(nach);
		 if(svon==null)	// Ausgangsstatus dieses Namens noch nicht vorhanden?
		 { 
		   svon=new S(von);	// neuen Status mit dem Namen erstellen...
		   alleZustaende.put(von,svon);	// ...und in Zustandsliste geben.
		  }
		 if(snach==null)		// dasselbe mit dem Zielstatus
		 {
		 	 snach=new S(nach);
		 	 alleZustaende.put(nach,snach);
		 	}
		 this.v(input,svon,snach,output);  // Die andere Verknpfungsmethode aufrufen.
	}
	
	/** Verknpft zwei Statusobjekte miteinander, bernimmt jedoch als Eingabe ein char */
	public void v(char input,S von,S nach, String output)
	{
		this.v(""+input,von,nach,output);
	}
	/** Verknpft zwei durch Namen referenzierte Statusobjekte miteinander, bernimmt jedoch
		* als Eingabe ein char */
	public void v(char input,String von, String nach, String output)
	{
		this.v(""+input,von,nach,output);
	}
	
	
	/** gehe() wechselt den aktuellen Zustand aufgrund der bergebenen Eingabe
		* @param input Eingabestring 
		* @return neuer Status */
	public S gehe(String input)
	{
		 aktZustand = aktZustand.geheZu(input);
		 return aktZustand;
	}
	
	/** Wechselt den aktuellen Zustand; bernimmt als Eingabe ein char */
	public S gehe(char input)
	{
	   return gehe(""+input);
	 }
	
	
	/** Lt den Automaten ablaufen. Vorsicht: bei zyklischen Graphen ohne "Enden"(Sackgassen)
		* ist dies eine Endlosschleife! */
	public void los()
	{
		Keyboard kb = new Keyboard();  // Liest Tastdrcke ein
		while(!aktZustand.istEnde())		// solange es noch Mglichkeiten gibt, weiterzukommen:
		{
			 gehe((char)kb.getkey());			// Taste einlesen und Zustand wechseln
		}
	}
}

	
				
	
	
		 
		 
		
	